package test_data

import (
	"github.com/gin-gonic/gin"
	"testData/model/response"
	test_data "testData/repository/test.data"
	"testData/request"
	"testData/utils"
)

type IChartSelectionService interface {
}

type ChartSelectionService struct{}

func InitChartSelectionService() *ChartSelectionService {
	return &ChartSelectionService{}
}

// @Tags 数据分析平台
// @Summary 查询图表-产品选项
// @Security ApiKeyAuth
// @accept application/json
// @Produce application/json
// @Param data body request.ChartSelection true "查询参数"
// @Success 200 {string} string "{"success":true,"data":{},"msg":"操作成功"}"
// @Router /testData/chart/productSelection [post]
func (S *ChartSelectionService) ProductSelection(c *gin.Context) {
	r := request.ChartSelection{}
	_ = c.ShouldBind(&r)
	if msg, ok := utils.ValidateInfo2CN(r); !ok {
		response.FailWithMessage(msg, c)
		return
	}
	response.OkWithData(test_data.ProductSelection(&r), c)
}

// @Tags 数据分析平台
// @Summary 查询图表-批次选项
// @Security ApiKeyAuth
// @accept application/json
// @Produce application/json
// @Param data body request.ChartSelection true "查询参数"
// @Success 200 {string} string "{"success":true,"data":{},"msg":"操作成功"}"
// @Router /testData/chart/lotSelection [post]
func (S *ChartSelectionService) LotSelection(c *gin.Context) {
	r := request.ChartSelection{}
	_ = c.ShouldBind(&r)
	if msg, ok := utils.ValidateInfo2CN(r); !ok {
		response.FailWithMessage(msg, c)
		return
	}
	response.OkWithData(test_data.LotSelection(&r), c)
}

// @Tags 数据分析平台
// @Summary 查询图表-PBI选项
// @Security ApiKeyAuth
// @accept application/json
// @Produce application/json
// @Param data body request.ChartSelection true "查询参数"
// @Success 200 {string} string "{"success":true,"data":{},"msg":"操作成功"}"
// @Router /testData/chart/pbiSelection [post]
func (S *ChartSelectionService) PBISelection(c *gin.Context) {
	r := request.ChartSelection{}
	_ = c.ShouldBind(&r)
	if msg, ok := utils.ValidateInfo2CN(r); !ok {
		response.FailWithMessage(msg, c)
		return
	}
	response.OkWithData(test_data.PBISelection(&r), c)
}
