package initialization

import (
	"github.com/spf13/viper"
	"gorm.io/driver/mysql"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"
	"testData/global"
	"time"
)

func InitBaseDB() {
	user := viper.GetString("Basedb.user")
	password := viper.GetString("Basedb.password")
	host := viper.GetString("Basedb.host")
	dbname := viper.GetString("Basedb.dbname")
	charset := viper.GetString("Basedb.charset")
	dsn := user + ":" + password + "@tcp(" + host + ")/" + dbname + "?charset=" + charset + "&parseTime=True&loc=Local"
	mysqlConfig := mysql.Config{
		DSN:                       dsn,   // DSN data source name
		DefaultStringSize:         255,   // string 类型字段的默认长度
		DisableDatetimePrecision:  true,  // 禁用 datetime 精度，MySQL 5.6 之前的数据库不支持
		DontSupportRenameIndex:    true,  // 重命名索引时采用删除并新建的方式，MySQL 5.7 之前的数据库和 MariaDB 不支持重命名索引
		DontSupportRenameColumn:   true,  // 用 `change` 重命名列，MySQL 8 之前的数据库和 MariaDB 不支持重命名列
		SkipInitializeWithVersion: false, // 根据版本自动配置
	}
	if db, err := gorm.Open(mysql.New(mysqlConfig), &gorm.Config{
		DisableForeignKeyConstraintWhenMigrating: true,
		Logger:                                   logger.Default.LogMode(logger.Silent),
	}); err != nil {
		global.Log.Errorf("%s", err)
		panic(err)
	} else {
		sqlDB, _ := db.DB()
		sqlDB.SetMaxIdleConns(10)           // 设置空闲连接池中连接的最大数量
		sqlDB.SetMaxOpenConns(100)          // 设置打开数据库连接的最大数量
		sqlDB.SetConnMaxLifetime(time.Hour) // 设置了连接可复用的最大时间
		global.BaseDB = db
	}
}
